#ifndef _HATPTASK_H_
#define _HATPTASK_H_

#include <string>
#include <vector>

#include <hatp_msgs/constants.h>
#include <hatp_msgs/Task.h>

enum TaskType{action, method};

class hatpTask;
typedef std::shared_ptr<hatpTask> hatpTaskPtr;
typedef std::vector<hatpTaskPtr> hatpTaskSet;
typedef std::vector<unsigned int> hatpTaskIDSet;
typedef std::map<unsigned int, hatpTaskPtr> hatpTaskMap;

class hatpPlan;

/** Represent a task.
 * Can be an action or a method.
 */
class hatpTask {
private:
	hatpPlan& plan;

	unsigned int id;
	TaskType type;

	std::string taskName;
	std::vector<std::string> agents;
	std::vector<std::string> parameters;

	double cost;
	double startTime;
	double duration;
	double endTime;

	unsigned int parent;
	std::vector<unsigned int> subnodes;

	std::vector<unsigned int> streamPredecessors;
	std::vector<unsigned int> streamSuccessors;

	std::vector<unsigned int> treePredecessors;
	std::vector<unsigned int> treeSuccessors;

	hatpTask();

public:

	/** Constructor of a node.
	 * @param id The ID of the node
	 * @param type The type of the node
	 * @param name The name of this task
	 * @param agents The list of agents
	 * @param params The list of parameters
	 * @param start The start time
	 * @param duration The end time
	 */
//	hatpTask(unsigned int id, TaskType type, std::string name, std::vector<std::string> agents, std::vector<std::string > params, double cost, double start, double duration);

	/** Constructor from a hatp_msgs::Task
	 * @param task Task to parse from ROS
	 */
	hatpTask(hatpPlan& plan, const hatp_msgs::Task& task);

	/** Get the node ID.
	 * @return The ID of the current node
	 */
	unsigned int getID() const;

	/** Get the node type.
	 * @return The type of the current task
	 */
	TaskType getType() const;

	/** Get the node type.
	 * @return True if this task is an action (isMethod will return false)
	 */
	bool isAction() const;

	/** Get the node type.
	 * @return True if this task is an action (isMethod will return false)
	 */
	bool isMethod() const;

	/** Get the node name.
	 * @return The name of the current task represented by this node
	 */
	std::string getName() const;

	/** Get the node parameters
	 * @return The list of parameters of the current task
	 */
	std::vector<std::string> getParameters() const;

	/** Get the agents list involved in the current task.
	 * @return The list of all the agents involved in this task
	 */
	std::vector<std::string> getAgents() const;

	/** Get the cost
	 * @return The cost of this action
	 */
	double getCost() const;

	/** Get the node start time.
	 * @return The time of the expected action start
	 */
	double getStartTime() const;

	/** Get the node duration.
	 * @return The expected duration for the action
	 */
	double getDuration() const;

	/** Get the node end time.
	 * @return The time of the expected action end
	 */
	double getEndTime() const;

	/** Get the parent of this task
	 * @return The ID of the parent of the current task, returns 0 if it does not exsit
	 */
	unsigned int getParent() const;

	/** Get the subnodes list.
	 * @return The list of subnodes of the current node
	 */
	std::vector<unsigned int> getSubNodesIds() const;

	/** Get the subnodes list.
	 * @return The list of subnodes of the current node
	 */
	hatpTaskSet getSubNodes() const;

	/** Get the list of predecessors (in tree).
	 * @return the list of nodes predecessors of the current one in the tree
	 */
	std::vector<unsigned int> getTreePredecessorsIds() const;

	/** Get the list of predecessors (in tree).
	 * @return the list of nodes predecessors of the current one in the tree
	 */
	hatpTaskSet getTreePredecessors() const;

	/** Get the list of successors (in tree).
	 * @return the list of nodes successors of the current one in the tree
	 */
	std::vector<unsigned int> getTreeSuccessorsIds() const;

	/** Get the list of successors (in tree).
	 * @return the list of nodes successors of the current one in the tree
	 */
	hatpTaskSet getTreeSuccessors() const;

	/** Get the list of predecessors (in streams).
	 * @return the list of nodes predecessors of the current one in the streams
	 */
	std::vector<unsigned int> getStreamPredecessorsIds() const;

	/** Get the list of predecessors (in streams).
	 * @return the list of nodes predecessors of the current one in the streams
	 */
	hatpTaskSet getStreamPredecessors() const;

	/** Get the list of successors (in streams).
	 * @return the list of nodes successors of the current one in the streams
	 */
	std::vector<unsigned int> getStreamSuccessorsIds() const;

	/** Get the list of successors (in streams).
	 * @return the list of nodes successors of the current one in the streams
	 */
	hatpTaskSet getStreamSuccessors() const;

	/** Set the parent of this task.
	 * @param n The parent ID
	 */
	void setParent(unsigned int n);

	/** Add a node the the subnodes list.
	 * @param n the ID of the node to add to the subnodes list
	 */
	void addSubNode(unsigned int n);

	/** Add a predecessor node to the current node (in the tree).
	 * @param n The ID of the node to add to the Tree Predecessors list
	 */
	void addTreePredecessor(unsigned int n);

	/** Add a successor ndoe to the current node (in the tree).
	 * @param n The ID of the node to add to the Tree Successors list
	 */
	void addTreeSuccessor(unsigned int n);

	/** Add a predecessor to the current node (in the streams).
	 * @param n The ID of the node to add to Streams Predecessors list
	 */
	void addStreamPredecessor(unsigned int n);

	/** Add a successor to the current node (in the streams).
	 * @param n The ID of the node to add to the Streams Successors list
	 */
	void addStreamSuccessor(unsigned int n);

	/** Return the name and the parameters.
	 * For instance: Task(param1, param2, ...)
	 * @return The string describing this task
	 */
	std::string getDescription() const;

	/** Get the node under a text form (for printing).
	 * @return A string containing the node under a text form
	 */
	std::string toString() const;

};

#endif //_HATPTASK_H_
