#include <libhatp/hatpStream.h>

#include <sstream>
#include <vector>

#include <libhatp/hatpPlan.h>

hatpStream::hatpStream(hatpPlan& plan, std::vector<hatp_msgs::StreamNode> const& streams): plan(plan), maxRank(-1){
	std::vector<hatp_msgs::StreamNode>::const_iterator it;
	for(it=streams.begin(); it!=streams.end(); ++it){
		if(!plan.hasTask(it->taskId)){
			std::cout<<"WARNING: Unknown task id: "<<it->taskId<<std::endl;
			continue;
		}
		hatpTaskPtr task=plan.getTask(it->taskId);
		
		std::vector<uint32_t>::const_iterator it_ids;
		for(it_ids=it->predecessors.begin(); it_ids!=it->predecessors.end(); ++it_ids){
			if(!plan.hasTask(*it_ids)) continue;
			task->addStreamPredecessor(*it_ids);
		}
		for(it_ids=it->successors.begin(); it_ids!=it->successors.end(); ++it_ids){
			if(!plan.hasTask(*it_ids)) continue;
			task->addStreamSuccessor(*it_ids);
		}
	}
	
	
	//Now that the list of predecessors/successors is initialised we can compute the ranks
	hatpTaskList& temp=plan.getTaskList();
	hatpTaskList::const_iterator itTemp;
	for(itTemp = temp.begin() ; itTemp != temp.end() ; itTemp++) {
		if(itTemp->second->getType()==action){
			computeNodeRank((*itTemp).second);
		}
	}
	
	//Finally fill in the stream for each agent
	int rank;
	for(rank=0; rank<=maxRank; ++rank){
		hatpTaskSet tasks=getNodesOfRank(rank);
		hatpTaskSet::const_iterator it_task;
		for(it_task=tasks.begin(); it_task!=tasks.end(); ++it_task){
			std::vector<std::string> agents=(*it_task)->getAgents();
			std::vector<std::string>::const_iterator it_agent;
			for(it_agent=agents.begin(); it_agent!=agents.end(); ++it_agent){
				this->streams[*it_agent].push_back(*it_task);
			}
		}
	}
}

void hatpStream::computeNodeRank(hatpTaskPtr cnode){
	if(getRankOfNode(cnode->getID()) > -1) return;
	if(cnode->getType()==method) return;
	int cRank = 0;
	std::vector<unsigned int> pred = cnode->getStreamPredecessorsIds();
	for(unsigned int i = 0 ; i < pred.size() ; i++) {
		hatpTaskPtr ctemp = plan.getTask(pred[i]);
		computeNodeRank(ctemp);
		if(getRankOfNode(ctemp->getID()) >= cRank) cRank = getRankOfNode(ctemp->getID()) + 1;
	}
	ranks.insert(std::make_pair(cnode->getID(), cRank));
	if(maxRank < (cRank)) maxRank = cRank;
}

int hatpStream::getRankOfNode(unsigned int n) const {
	if(ranks.find(n)!=ranks.end()){
		return ranks.at(n);
	}
	return -1;
}

hatpTaskSet hatpStream::getNodesOfRank(int n) const{
	hatpTaskSet out;

	if(n < 0){
		return out;
	}

	std::map<unsigned int, int>::const_iterator it;
	
	for(it = ranks.begin() ; it != ranks.end() ; ++it) {
		if(it->second==n){
			out.push_back(plan.getTask(it->first));
		}
	}
	return out;
}

unsigned int hatpStream::getMaxRank() const{
	return maxRank;
}

hatpTaskSet const& hatpStream::getStreamForAgent(std::string const& agent) const{
	if(streams.find(agent)==streams.end()){
		throw std::runtime_error("Requesting the stream for agent "+agent+" which does not exist");
	}
	return streams.find(agent)->second;
}

std::map< std::string, hatpTaskSet > const& hatpStream::getAgentsStreams() const{
	return streams;
}

std::string hatpStream::toString() const{
	std::stringstream out;
	
	out<<"<----- BEGIN STREAMS ----->"<<std::endl;
	out<<"Task  -  Rank"<<std::endl;
	for(auto i: ranks){
		out<<" "<<std::left<<std::setw(5)<<i.first<<"-"<<std::right<<std::setw(5)<<i.second<<std::endl;
	}
	
// 	out<<"=== Agents streams ==="<<std::endl;
// 	std::map< std::string, hatpTaskSet >::const_iterator it_streams;
// 	hatpTaskSet::const_iterator it_actions;
// 	for(it_streams=streams.begin(); it_streams!=streams.end(); ++it_streams){
// 		out<<"AGENT: "<<it_streams->first<<std::endl;
// 		for(it_actions=it_streams->second.begin(); it_actions!=it_streams->second.end(); ++it_actions){
// 			out<<"\t"<<(*it_actions)->getDescription()<<std::endl;
// 		}
// 	}
	
	return out.str();
}
